--[[ NavHelper by MrInfernal  v0.9

GUI to improve usage of MQ2Nav and streamline travel
Allows the user to search by zone full name, short name, or ID and returns
a selectable table. Click a zone and you can Start, Pause, or Cancel travel.

There are also buttons to check for teleport locations near your selected zone.
Self cast has the current character cast, while Group sends a DanNet command
]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- Import the zone and port files
local zones = require('navzones')
local ports = require('zoneports')

-- Set default values
local zoneSearch = ""       -- when 'Search' is clicked
local results = {}          -- results from zoneLookup()
local portresults = {}      -- when 'Check' is clicked [Searches the zone for nearby ports]
local destZone = ""         -- the zone chosen after Searching
local portChecked = false   -- keep port results table from showing until data is available
local portSpell = ""        -- the port spell to cast
local groupTravel = true

-- Add druids and wizards to a DanNet channel called porters
mq.cmd('/squelch /dgex dru /djoin porters')
mq.cmd('/squelch /dgex wiz /djoin porters')

-- takes the input field and searches through the navzones for matches
-- can match the full name, short name, or id
local function zoneLookup(input)
    local results = {}

    -- Convert the input to lowercase for case-insensitive comparison
    local lowerInput = string.lower(input)

    -- Checks for zones that match the input
    for _, zone in ipairs(zones) do
        local lowerName = string.lower(zone.name)
        local lowerShortName = string.lower(zone.shortname)

        if string.find(lowerName, lowerInput) or
           string.find(lowerShortName, lowerInput) or
           tostring(zone.id) == input then
            table.insert(results, zone)
        end
    end

    return results
end

-- checks the selected zone for nearby port spell locations
-- zoneports have the direct location of the spell, as well as the surrounding zones
local function portCheck(input)
    local portresults = {}
    local resultsFound = false

    -- Checks for ports near our destination
    for _, port in ipairs(ports) do
        if string.find(port.loc, input) then
            table.insert(portresults, port)
            resultsFound = true
        end
    end

    if not resultsFound then
        for _, port in ipairs(ports) do
            for _, conn in ipairs(port.connections) do
                if string.find(conn, input) then
                    table.insert(portresults, port)
                end
            end
        end
    end

    return portresults
end

-- main function to run the window
function navHelper(open)
    -- We specify a default position/size in case there's no data in the .ini file.
    -- We only do it to make the demo applications a little more welcoming, but typically this isn't required.
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 650, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)
    imgui.SetNextWindowSize(1550, 850, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("NavHelper", open)

    if not show then
        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Beginning of window elements
    imgui.Text("Search by zone name, short name, or ID")
    zoneSearch = ImGui.InputText('', zoneSearch)

    imgui.SameLine()
    if ImGui.Button("Search") then
        results = zoneLookup(zoneSearch)
        portChecked = false
    end
    imgui.SameLine()
    if ImGui.Button("Reset") then
        results = {} -- Reset the results
        zoneSearch = "" -- Clear the search input
        portChecked = false -- Clear the port info display
    end

    imgui.Text('\n')
    imgui.Text('Travel: ')
    imgui.SameLine()

    --Buttons to Start, Pause, or Cancel travel
    -- Add a checkbox and store its state in the 'groupTravel' variable [Default is true]
    groupTravel = imgui.Checkbox("Group", groupTravel)
    imgui.SameLine()
    if ImGui.Button("Start") then
        if destZone ~= "" then
            if groupTravel then
                mq.cmd("/dgae /travelto ", destZone)
            else
                mq.cmd("/travelto ", destZone)
            end
        else
            print('No zone selected!')
        end
    end
    imgui.SameLine()
    if ImGui.Button("Pause") then
        if groupTravel then
            mq.cmd("/dgae /nav pause")
        else
            mq.cmd("/nav pause")
        end
    end
    imgui.SameLine()
    if ImGui.Button("Stop") then
        if groupTravel then
            mq.cmd("/dgae /travelto stop")
        else
            mq.cmd("/travelto stop")
        end
    end

    -- Buttons to Check for ports. Self Cast means the selected character will cast
    -- Group Cast will send a DanNet message to the porter channel
    -- Also pauses CWTN plugins
    -- [To do: Unpause the CWTN plugins after zoning]
    -- Create a function to handle the group teleporting
    function boundPort(arg, input)
        if arg == "group" then
            print(string.format('Casting "%s"!', input))
            mq.cmd('/dgex porters /dru pause on')
            mq.cmdf('/dgex porters /casting "%s" gem5', input)
        else
            mq.cmdf('/casting "%s" gem5', arg)
        end
    end

    imgui.SameLine(300)
    imgui.Text('Ports: ')
    imgui.SameLine()
    if ImGui.Button("Check") then
        if destZone ~= "" then
            portChecked = true
            portresults = portCheck(destZone)
        else
            print('No zone selected!')
        end
    end
    imgui.SameLine()
    if ImGui.Button("Self Cast") then
        if portSpell == "" then
            print ('No port location selected!')
        else
            mq.cmdf('/casting "%s" gem5', portSpell)
        end
    end
    imgui.SameLine()
    if ImGui.Button("Group Cast") then
        boundPort(arg, ...)
    end    

    ImGui.Separator()
    -- Create table headings
    imgui.Text('Zone Name')
    ImGui.SameLine(240)
    imgui.Text('Short Name')
    ImGui.SameLine(400)
    imgui.Text('Zone ID')
    imgui.Text('\n')

    -- Iterate over results and create a selectable row for each
    for _, result in ipairs(results) do
        local isSelected = destZone == result.shortname
        if ImGui.Selectable(result.name, isSelected, ImGuiSelectableFlags.SpanAllColumns) then
            -- Set the selected zone to pass to other functions
            destZone = result.shortname
        end

        -- Display short name in the second column
        ImGui.SameLine(240)
        ImGui.Text(result.shortname)

        -- Display ID in the third column
        ImGui.SameLine(400)
        ImGui.Text(tostring(result.id))
    end

    --Port results. Will only show when results are available
    if portChecked then
        imgui.Text('\n')
        ImGui.Separator()
        imgui.Text('Nearby port locations')
        imgui.Text('\n')
        -- Create table headings
        imgui.Text('Level')
        ImGui.SameLine(60)
        imgui.Text('Spell Name')
        ImGui.SameLine(260)
        imgui.Text('Location')
        ImGui.SameLine(380)
        imgui.Text('Target')
        ImGui.SameLine(460)
        imgui.Text('Class')
        imgui.Text('\n')

        -- Iterate over results and create a selectable row for each
        for _, result2 in ipairs(portresults) do
            if portresults == {} then
                imgui.Text('No port locations found!')
            else
                local label = tostring(result2.level) .. "           " .. result2.name
                local isSelected2 = portSpell == result2.name
                if ImGui.Selectable(label, isSelected2, ImGuiSelectableFlags.SpanAllColumns) then
                    portSpell = result2.name  -- Set the selected spell
                end
                
                ImGui.SameLine(260)
                imgui.Text(result2.loc)
                ImGui.SameLine(380)
                imgui.Text(result2.target)
                ImGui.SameLine(460)
                imgui.Text(result2.class)
            end
        end
    end

    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

-- Creates a slash command for porting. Allows creation of hotkeys
mq.bind('/nh', boundPort)

local openGUI = true

ImGui.Register('NavHelper', function()
    openGUI = navHelper(openGUI)
end)

while openGUI do
    mq.delay(1000) -- equivalent to '1s'
end